@@ -99,7 +99,7 @@ static GParamSpec *menu_props[N_PROPERTIES] = { NULL, };
 
 
 
-G_DEFINE_TYPE (GarconGtkMenu, garcon_gtk_menu, GTK_TYPE_MENU)
+G_DEFINE_TYPE_WITH_PRIVATE (GarconGtkMenu, garcon_gtk_menu, GTK_TYPE_MENU)
 
 
 
@@ -109,8 +109,6 @@ garcon_gtk_menu_class_init (GarconGtkMenuClass *klass)
   GObjectClass   *gobject_class;
   GtkWidgetClass *gtkwidget_class;
 
-  g_type_class_add_private (klass, sizeof (GarconGtkMenuPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = garcon_gtk_menu_finalize;
   gobject_class->get_property = garcon_gtk_menu_get_property;
@@ -206,7 +204,7 @@ garcon_gtk_menu_class_init (GarconGtkMenuClass *klass)
 static void
 garcon_gtk_menu_init (GarconGtkMenu *menu)
 {
-  menu->priv = G_TYPE_INSTANCE_GET_PRIVATE (menu, GARCON_GTK_TYPE_MENU, GarconGtkMenuPrivate);
+  menu->priv = garcon_gtk_menu_get_instance_private (menu);
 
   menu->priv->show_generic_names = FALSE;
   menu->priv->show_menu_icons = TRUE;
@@ -712,7 +710,7 @@ garcon_gtk_menu_load_icon (const gchar *icon_name)
 
 
 static GtkWidget*
-garcon_gtk_menu_create_menu_item (GarconGtkMenu *menu,
+garcon_gtk_menu_create_menu_item (gboolean     show_menu_icons,
                                   const gchar *name,
                                   const gchar *icon_name)
 {
@@ -734,7 +732,7 @@ garcon_gtk_menu_create_menu_item (GarconGtkMenu *menu,
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5f);
 #endif
 
-  if (menu->priv->show_menu_icons)
+  if (show_menu_icons)
     {
       image = garcon_gtk_menu_load_icon (icon_name);
       gtk_widget_show (image);
@@ -755,30 +753,20 @@ garcon_gtk_menu_create_menu_item (GarconGtkMenu *menu,
 
 
 
-static GtkWidget*
-garcon_gtk_menu_add_actions (GarconGtkMenu  *menu,
-                             GarconMenuItem *menu_item,
-                             GList          *actions,
-                             const gchar    *parent_icon_name)
+static void
+garcon_gtk_menu_pack_actions_menu (GtkWidget      *menu,
+                                   GarconMenuItem *menu_item,
+                                   GList          *actions,
+                                   const gchar    *parent_icon_name,
+                                   gboolean        show_menu_icons)
 {
-  GtkWidget *submenu, *mi;
   GList     *iter;
+  GtkWidget *mi;
 
-  submenu = gtk_menu_new ();
-  gtk_menu_set_reserve_toggle_size (GTK_MENU (submenu), FALSE);
-
-  /* Add the parent item again, this time something the user can click to execute */
-  mi = garcon_gtk_menu_create_menu_item (menu, garcon_menu_item_get_name (menu_item), parent_icon_name);
-  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mi);
-  /* we need to store the GarconGtkMenu with this item so we can
-   * use it if the user wants to edit a menu item */
-  g_object_set_data (G_OBJECT (mi), "GarconGtkMenu", menu);
-  g_signal_connect (G_OBJECT (mi), "activate",
-                    G_CALLBACK (garcon_gtk_menu_item_activate), menu_item);
-  gtk_widget_show (mi);
+  gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
 
   /* Add all the individual actions to the menu */
-  for (iter = g_list_first(actions); iter != NULL; iter = g_list_next (iter))
+  for (iter = g_list_first (actions); iter != NULL; iter = g_list_next (iter))
     {
       GarconMenuItemAction *action = garcon_menu_item_get_action (menu_item, iter->data);
       const gchar          *action_icon_name;
@@ -795,11 +783,11 @@ garcon_gtk_menu_add_actions (GarconGtkMenu  *menu,
           action_icon_name = parent_icon_name;
         }
 
-      mi = garcon_gtk_menu_create_menu_item (menu,
+      mi = garcon_gtk_menu_create_menu_item (show_menu_icons,
                                              garcon_menu_item_action_get_name (action),
                                              action_icon_name);
 
-      gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mi);
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
                         G_CALLBACK (garcon_gtk_menu_item_action_activate), action);
       /* we need to store the parent associated with this item so we can
@@ -807,6 +795,35 @@ garcon_gtk_menu_add_actions (GarconGtkMenu  *menu,
       g_object_set_data (G_OBJECT (action), "GarconMenuItem", menu_item);
       gtk_widget_show (mi);
     }
+}
+
+
+
+static GtkWidget*
+garcon_gtk_menu_add_actions (GarconGtkMenu  *menu,
+                             GarconMenuItem *menu_item,
+                             GList          *actions,
+                             const gchar    *parent_icon_name)
+{
+  GtkWidget *submenu, *mi;
+
+  submenu = gtk_menu_new ();
+
+  /* Add the parent item again, this time something the user can click to execute */
+  mi = garcon_gtk_menu_create_menu_item (menu->priv->show_menu_icons,
+                                         garcon_menu_item_get_name (menu_item),
+                                         parent_icon_name);
+  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mi);
+
+  /* we need to store the GarconGtkMenu with this item so we can
+   * use it if the user wants to edit a menu item */
+  g_object_set_data (G_OBJECT (mi), "GarconGtkMenu", menu);
+  g_signal_connect (G_OBJECT (mi), "activate",
+                    G_CALLBACK (garcon_gtk_menu_item_activate), menu_item);
+  gtk_widget_show (mi);
+
+  garcon_gtk_menu_pack_actions_menu (submenu, menu_item, actions,
+                                     parent_icon_name, menu->priv->show_menu_icons);
 
   return submenu;
 }
@@ -863,7 +880,7 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
             icon_name = "applications-other";
 
           /* build the menu item */
-          mi = garcon_gtk_menu_create_menu_item (menu, name, icon_name);
+          mi = garcon_gtk_menu_create_menu_item (menu->priv->show_menu_icons, name, icon_name);
           gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
 
           /* if the menu item has actions such as "Private browsing mode"
@@ -944,7 +961,7 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
                 icon_name = "applications-other";
 
               /* build the menu item */
-              mi = garcon_gtk_menu_create_menu_item (menu, name, icon_name);
+              mi = garcon_gtk_menu_create_menu_item (menu->priv->show_menu_icons, name, icon_name);
 
               gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
               gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), submenu);
@@ -1186,7 +1203,7 @@ garcon_gtk_menu_set_show_tooltips (GarconGtkMenu *menu,
  * garcon_gtk_menu_get_show_tooltips:
  * @menu  : A #GarconGtkMenu
  *
- * Return value: if descriptions are shown in tooltip
+ * Return value: Whether descriptions are shown in the tooltip.
  **/
 gboolean
 garcon_gtk_menu_get_show_tooltips (GarconGtkMenu *menu)
@@ -1195,6 +1212,8 @@ garcon_gtk_menu_get_show_tooltips (GarconGtkMenu *menu)
   return menu->priv->show_tooltips;
 }
 
+
+
 /**
  * garcon_gtk_menu_set_show_desktop_actions:
  * @menu  : A #GarconGtkMenu
@@ -1222,7 +1241,7 @@ garcon_gtk_menu_set_show_desktop_actions (GarconGtkMenu *menu,
  * garcon_gtk_menu_get_show_desktop_actions:
  * @menu  : A #GarconGtkMenu
  *
- * Return value: if the desktop actions in a submenu
+ * Return value: Whether desktop actions are shown in a submenu.
  **/
 gboolean
 garcon_gtk_menu_get_show_desktop_actions (GarconGtkMenu *menu)
@@ -1232,10 +1251,39 @@ garcon_gtk_menu_get_show_desktop_actions (GarconGtkMenu *menu)
 }
 
 
+
+/**
+ * garcon_gtk_menu_get_desktop_actions_menu:
+ * @item  : A #GarconMenuItem
+ *
+ * Return value: a #GtkMenu holding all actions described in the desktop file
+ * as menu items. Note that application icons are never shown on the
+ * action menu items.
+ **/
+GtkMenu *
+garcon_gtk_menu_get_desktop_actions_menu (GarconMenuItem *item)
+{
+  GtkWidget   *submenu = gtk_menu_new ();
+  GList       *actions = NULL;
+  const gchar *parent_icon_name;
+  gboolean     show_menu_icons = FALSE;
+
+  actions = garcon_menu_item_get_actions (item);
+  g_return_val_if_fail (actions != NULL, NULL);
+
+  parent_icon_name = garcon_menu_item_get_icon_name (item);
+
+  garcon_gtk_menu_pack_actions_menu (submenu, item, actions, parent_icon_name, show_menu_icons);
+
+  return GTK_MENU (submenu);
+}
+
+
+
 /**
  * garcon_gtk_menu_set_right_click_edits:
  * @menu  : A #GarconGtkMenu
- * @enable_right_click_edits : Toggle showing wether to launch an editor
+ * @enable_right_click_edits : Toggle showing whether to launch an editor
  * when the menu is clicked with the secondary mouse button.
  *
  **/
@@ -1260,7 +1308,7 @@ garcon_gtk_menu_set_right_click_edits (GarconGtkMenu *menu,
  * garcon_gtk_menu_get_right_click_edits:
  * @menu  : A #GarconGtkMenu
  *
- * Return value: if an editor will be launched on secondary mouse clicks.
+ * Return value: Whether an editor will be launched on secondary mouse clicks
  **/
 gboolean
 garcon_gtk_menu_get_right_click_edits (GarconGtkMenu *menu)
